@@ -1,11 +1,10 @@
 ;;; pi-coding-agent-core.el --- Core functionality for pi-coding-agent -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2025 Daniel Nouri
+;; Copyright (C) 2026 Daniel Nouri
 
 ;; Author: Daniel Nouri <daniel.nouri@gmail.com>
 ;; Maintainer: Daniel Nouri <daniel.nouri@gmail.com>
-;; URL: https://github.com/dnouri/pi.el
-;; Keywords: ai llm ai-pair-programming tools
+;; URL: https://github.com/dnouri/pi-coding-agent
 
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -95,6 +94,12 @@ CALLBACK is called with the response plist when received."
     (puthash id callback pending)
     (process-send-string process (pi-coding-agent--encode-command full-command))))
 
+(defun pi-coding-agent--send-extension-ui-response (process response)
+  "Send extension UI RESPONSE to pi PROCESS.
+RESPONSE must include the original :id from the request, as pi uses
+this to match responses to pending promises."
+  (process-send-string process (pi-coding-agent--encode-command response)))
+
 (defun pi-coding-agent--rpc-sync (process command &optional timeout)
   "Send COMMAND to pi PROCESS synchronously, returning the response.
 Blocks until response is received or TIMEOUT seconds elapse.
@@ -166,13 +171,21 @@ containing EVENT, then clears this process's pending requests table."
                pending)
       (clrhash pending))))
 
+(defvar pi-coding-agent-extra-args nil
+  "Extra arguments to pass to the pi command.
+A list of strings that will be appended to the base command.
+
+Example: (setq pi-coding-agent-extra-args \\='(\"-e\" \"/path/to/ext.ts\"))
+
+This is useful for testing extensions or passing additional flags.")
+
 (defun pi-coding-agent--start-process (directory)
   "Start pi RPC process in DIRECTORY.
 Returns the process object."
   (let ((default-directory directory))
     (make-process
      :name "pi"
-     :command '("pi" "--mode" "rpc")
+     :command `("pi" "--mode" "rpc" ,@pi-coding-agent-extra-args)
      :connection-type 'pipe
      :filter #'pi-coding-agent--process-filter
      :sentinel #'pi-coding-agent--process-sentinel)))
@@ -181,8 +194,14 @@ Returns the process object."
 
 (defvar-local pi-coding-agent--status 'idle
   "Current status of the pi session (buffer-local in chat buffer).
-One of: `idle', `sending', `streaming', `compacting'.
-This is the single source of truth for session activity state.")
+One of: `idle', `streaming', `compacting'.
+This is the single source of truth for session activity state.
+
+Status transitions are driven by events from pi:
+- `idle' -> `streaming' on agent_start
+- `streaming' -> `idle' on agent_end
+- `idle' -> `compacting' on auto_compaction_start
+- `compacting' -> `idle' on auto_compaction_end")
 
 (defvar-local pi-coding-agent--state nil
   "Current state of the pi session (buffer-local in chat buffer).
@@ -198,7 +217,7 @@ A plist with keys like :model, :thinking-level, :messages, etc.")
   "Return t if state should be verified with get_state.
 Verification is needed when:
 - State and timestamp exist
-- Session is idle (not streaming, sending, or compacting)
+- Session is idle (not streaming or compacting)
 - Timestamp is older than `pi-coding-agent--state-verify-interval' seconds."
   (and pi-coding-agent--state
        pi-coding-agent--state-timestamp
@@ -210,11 +229,22 @@ Verification is needed when:
   "Return t if VALUE represents JSON false."
   (eq value :false))
 
+(defun pi-coding-agent--json-null-p (value)
+  "Return t if VALUE represents JSON null.
+`json-parse-string' decodes JSON null as the keyword :null."
+  (eq value :null))
+
 (defun pi-coding-agent--normalize-boolean (value)
   "Convert JSON boolean VALUE to Elisp boolean.
 JSON true (t) stays t, JSON false (:false) becomes nil."
   (if (pi-coding-agent--json-false-p value) nil value))
 
+(defun pi-coding-agent--normalize-string-or-null (value)
+  "Return VALUE if it's a string, nil otherwise.
+Use when reading JSON fields that may be null or string.
+JSON null (:null) and non-strings become nil."
+  (and (stringp value) value))
+
 (defun pi-coding-agent--update-state-from-event (event)
   "Update status and state based on EVENT.
 Handles agent lifecycle, message events, and error/retry events.
