@@ -228,35 +228,10 @@ For backward search: go to current input (nil index)."
 
 ;;;; Input Mode
 
-(defun pi-coding-agent--input-mode-strip-metadata-keywords ()
-  "Remove markdown metadata font-lock keywords from the current buffer.
-Strips YAML, declarative, and pandoc metadata matchers that would
-otherwise highlight lines ending with `:' as metadata keys."
-  (font-lock-remove-keywords nil
-    (cl-remove-if-not
-     (lambda (kw)
-       (and (listp kw) (symbolp (car kw))
-            (string-match-p "metadata" (symbol-name (car kw)))))
-     markdown-mode-font-lock-keywords)))
-
 (define-derived-mode pi-coding-agent-input-mode text-mode "Pi-Input"
-  "Major mode for composing pi prompts.
-Defaults to plain `text-mode'.  Set
-`pi-coding-agent-input-markdown-highlighting' to non-nil for GFM
-syntax highlighting while preserving mode identity and keybindings."
+  "Major mode for composing pi prompts."
   :group 'pi-coding-agent
-  (when pi-coding-agent-input-markdown-highlighting
-    (gfm-mode)
-    (setq major-mode 'pi-coding-agent-input-mode)
-    (setq mode-name "Pi-Input")
-    (use-local-map pi-coding-agent-input-mode-map)
-    ;; Users see exactly what they type — never hide markup in input.
-    (setq-local markdown-hide-markup nil)
-    (pi-coding-agent--input-mode-strip-metadata-keywords))
   (setq-local header-line-format '(:eval (pi-coding-agent--header-line-string)))
-  ;; Reset inherited completions (text-mode adds ispell, etc.) — our
-  ;; input buffer should only offer slash commands, file refs, and paths.
-  (setq-local completion-at-point-functions nil)
   (add-hook 'completion-at-point-functions #'pi-coding-agent--command-capf nil t)
   (add-hook 'completion-at-point-functions #'pi-coding-agent--file-reference-capf nil t)
   (add-hook 'completion-at-point-functions #'pi-coding-agent--path-capf nil t)
@@ -266,24 +241,10 @@ syntax highlighting while preserving mode identity and keybindings."
 
 ;;;; Sending Prompts
 
-(defun pi-coding-agent--accept-input-text (text)
-  "Accept TEXT from input buffer state.
-Adds TEXT to history, resets history navigation, and clears input."
-  (pi-coding-agent--history-add text)
-  (setq pi-coding-agent--input-ring-index nil
-        pi-coding-agent--input-saved nil)
-  (erase-buffer))
-
-(defun pi-coding-agent--queue-followup-text (chat-buf text)
-  "Accept TEXT and enqueue it as a follow-up in CHAT-BUF."
-  (pi-coding-agent--accept-input-text text)
-  (with-current-buffer chat-buf
-    (pi-coding-agent--push-followup text)))
-
 (defun pi-coding-agent-send ()
   "Send the current input buffer contents to pi.
 Clears the input buffer after sending.  Does nothing if buffer is empty.
-If pi is busy (sending, streaming, or compacting), queues a local follow-up.
+If pi is busy (streaming or compacting), adds to local follow-up queue.
 The /compact command is handled locally; other slash commands sent to pi."
   (interactive)
   (let* ((text (string-trim (buffer-string)))
@@ -293,10 +254,18 @@ The /compact command is handled locally; other slash commands sent to pi."
     (cond
      ((string-empty-p text) nil)
      (busy
-      (pi-coding-agent--queue-followup-text chat-buf text)
+      (pi-coding-agent--history-add text)
+      (setq pi-coding-agent--input-ring-index nil
+            pi-coding-agent--input-saved nil)
+      (erase-buffer)
+      (with-current-buffer chat-buf
+        (pi-coding-agent--push-followup text))
       (message "Pi: Message queued (will send after current response)"))
      (t
-      (pi-coding-agent--accept-input-text text)
+      (pi-coding-agent--history-add text)
+      (setq pi-coding-agent--input-ring-index nil
+            pi-coding-agent--input-saved nil)
+      (erase-buffer)
       (with-current-buffer chat-buf
         (pi-coding-agent--prepare-and-send text))))))
 
@@ -352,15 +321,13 @@ cancels, the session remains intact."
 (defun pi-coding-agent--command-capf ()
   "Completion-at-point function for /commands in input buffer.
 Returns completion data when point is after / at start of buffer.
-Includes both built-in commands and commands from pi's `get_commands' RPC."
+Uses commands from pi's `get_commands' RPC."
   (when (and (eq (char-after (point-min)) ?/)
              (> (point) (point-min)))
     (let* ((start (1+ (point-min)))
            (end (point))
-           (builtin-names (mapcar #'car pi-coding-agent--builtin-commands))
-           (rpc-names (mapcar (lambda (cmd) (plist-get cmd :name))
-                              pi-coding-agent--commands))
-           (commands (delete-dups (append builtin-names rpc-names))))
+           (commands (mapcar (lambda (cmd) (plist-get cmd :name))
+                             pi-coding-agent--commands)))
       (list start end commands :exclusive 'no))))
 
 ;;;; Editor Features: File Reference (@)
@@ -519,29 +486,25 @@ Shows error message if RPC fails."
 
 (defun pi-coding-agent-queue-steering ()
   "Send current input as a steering message.
-When pi is sending or streaming, steering interrupts remaining tools.
+Steering messages are delivered after the current tool execution,
+interrupting any remaining tools.  Only works when agent is busy.
 Unlike normal sends, steering is NOT displayed locally - pi will echo
 it back via message_start at the correct position (after current
-assistant output completes).
-
-When compaction is in progress, steering text is queued as a local
-follow-up and sent after compaction completes."
+assistant output completes)."
   (interactive)
   (let ((text (string-trim (buffer-string))))
     (unless (string-empty-p text)
       (let ((chat-buf (pi-coding-agent--get-chat-buffer)))
         (when chat-buf
           (let ((status (buffer-local-value 'pi-coding-agent--status chat-buf)))
-            (cond
-             ((eq status 'idle)
-              (message "Pi: Nothing to interrupt - use C-c C-c to send"))
-             ((eq status 'compacting)
-              (pi-coding-agent--queue-followup-text chat-buf text)
-              (message "Pi: Steering queued (will send after compaction)"))
-             (t
-              (when (pi-coding-agent--send-steer-message text)
-                (pi-coding-agent--accept-input-text text)
-                (message "Pi: Steering message sent"))))))))))
+            (if (eq status 'idle)
+                (message "Pi: Nothing to interrupt - use C-c C-c to send")
+              (pi-coding-agent--history-add text)
+              (setq pi-coding-agent--input-ring-index nil
+                    pi-coding-agent--input-saved nil)
+              (erase-buffer)
+              (pi-coding-agent--send-steer-message text)
+              (message "Pi: Steering message sent"))))))))
 
 (defun pi-coding-agent-queue-followup ()
   "Queue current input as a follow-up message.
