@@ -184,21 +184,6 @@ on each delta - fontification happens at message end instead."
           (insert transformed)
           (set-marker pi-coding-agent--streaming-marker (point)))))))
 
-(defun pi-coding-agent--thinking-insert-position ()
-  "Return insertion position for thinking text.
-Prefers `pi-coding-agent--thinking-marker' when available so interleaved
-tool headers do not move the thinking insertion point."
-  (if (and pi-coding-agent--thinking-marker
-           (marker-position pi-coding-agent--thinking-marker))
-      (marker-position pi-coding-agent--thinking-marker)
-    (marker-position pi-coding-agent--streaming-marker)))
-
-(defun pi-coding-agent--clear-thinking-marker ()
-  "Detach and clear `pi-coding-agent--thinking-marker'."
-  (when (markerp pi-coding-agent--thinking-marker)
-    (set-marker pi-coding-agent--thinking-marker nil))
-  (setq pi-coding-agent--thinking-marker nil))
-
 (defun pi-coding-agent--display-thinking-start ()
   "Insert opening marker for thinking block (blockquote)."
   (when pi-coding-agent--streaming-marker
@@ -208,15 +193,10 @@ tool headers do not move the thinking insertion point."
         (save-excursion
           (goto-char (marker-position pi-coding-agent--streaming-marker))
           (insert "> ")
-          ;; Track thinking insertion separately so it stays anchored even if
-          ;; other block types (tool headers) interleave in the same message.
-          ;; Keep insertion-type nil so inserts at this exact point happen
-          ;; after the marker (we then advance it explicitly per delta).
-          (pi-coding-agent--clear-thinking-marker)
-          (setq pi-coding-agent--thinking-marker (copy-marker (point) nil)))))))
+          (set-marker pi-coding-agent--streaming-marker (point)))))))
 
 (defun pi-coding-agent--display-thinking-delta (delta)
-  "Display streaming thinking DELTA in the current thinking block.
+  "Display streaming thinking DELTA at the streaming marker.
 Transforms newlines to include blockquote prefix.
 Inhibits modification hooks to prevent expensive jit-lock fontification
 on each delta - fontification happens at message end instead."
@@ -227,10 +207,9 @@ on each delta - fontification happens at message end instead."
           (transformed (replace-regexp-in-string "\n" "\n> " delta)))
       (pi-coding-agent--with-scroll-preservation
         (save-excursion
-          (goto-char (pi-coding-agent--thinking-insert-position))
+          (goto-char (marker-position pi-coding-agent--streaming-marker))
           (insert transformed)
-          (when pi-coding-agent--thinking-marker
-            (set-marker pi-coding-agent--thinking-marker (point))))))))
+          (set-marker pi-coding-agent--streaming-marker (point)))))))
 
 (defun pi-coding-agent--display-thinking-end (_content)
   "End thinking block (blockquote).
@@ -240,10 +219,10 @@ CONTENT is ignored - we use what was already streamed."
     (let ((inhibit-read-only t))
       (pi-coding-agent--with-scroll-preservation
         (save-excursion
-          (goto-char (pi-coding-agent--thinking-insert-position))
+          (goto-char (marker-position pi-coding-agent--streaming-marker))
           ;; End blockquote with blank line
           (insert "\n\n")
-          (pi-coding-agent--clear-thinking-marker))))))
+          (set-marker pi-coding-agent--streaming-marker (point)))))))
 
 (defun pi-coding-agent--display-agent-end ()
   "Finalize agent turn: normalize whitespace, handle abort, process queue.
@@ -251,8 +230,6 @@ Note: status is set to `idle' by the event handler."
   ;; Reset per-turn state for clean next turn.
   (setq pi-coding-agent--local-user-message nil)
   (setq pi-coding-agent--streaming-tool-id nil)
-  (setq pi-coding-agent--in-thinking-block nil)
-  (pi-coding-agent--clear-thinking-marker)
   (let ((was-aborted pi-coding-agent--aborted))
     (let ((inhibit-read-only t))
       (pi-coding-agent--tool-overlay-finalize 'pi-coding-agent-tool-block-error)
@@ -431,8 +408,6 @@ Shows success or final failure with raw error."
   "Handle setStatus method from EVENT."
   (let ((key (plist-get event :statusKey))
         (text (plist-get event :statusText)))
-    (when text
-      (setq text (ansi-color-filter-apply text)))
     (if text
         (setq pi-coding-agent--extension-status
               (cons (cons key text)
@@ -487,7 +462,7 @@ Shown when the session starts without a configured model/API key."
 
 (defun pi-coding-agent--cleanup-on-kill ()
   "Clean up resources when chat buffer is killed.
-Also kills the linked input buffer and fontification cache buffers.
+Also kills the linked input buffer.
 
 Note: This runs from `kill-buffer-hook', which executes AFTER the kill
 decision is made.  For proper cancellation support, use `pi-coding-agent-quit'
@@ -497,7 +472,6 @@ which asks upfront before any buffers are touched."
       (pi-coding-agent--unregister-display-handler pi-coding-agent--process)
       (when (process-live-p pi-coding-agent--process)
         (delete-process pi-coding-agent--process)))
-    (pi-coding-agent--kill-fontify-buffers)
     (when (and pi-coding-agent--input-buffer (buffer-live-p pi-coding-agent--input-buffer))
       (let ((input-buf pi-coding-agent--input-buffer))
         (pi-coding-agent--set-input-buffer nil) ; break cycle before kill
@@ -548,9 +522,6 @@ Updates buffer-local state and renders display updates."
     ("message_start"
      (let* ((message (plist-get event :message))
             (role (plist-get message :role)))
-       ;; A new message starts a fresh rendering context.
-       (setq pi-coding-agent--in-thinking-block nil)
-       (pi-coding-agent--clear-thinking-marker)
        (pcase role
          ("user"
           ;; User message from pi - check if we displayed it locally
@@ -879,8 +850,7 @@ Uses `font-lock-face' to survive gfm-mode refontification."
            name))))))
 
 (defun pi-coding-agent--display-tool-start (tool-name args)
-  "Insert tool header for TOOL-NAME with ARGS and create pending overlay.
-Records the header-end position for later content insertion."
+  "Display header for tool TOOL-NAME with ARGS and create overlay."
   (let* ((header-display (pi-coding-agent--tool-header tool-name args))
          (path (pi-coding-agent--tool-path args))
          (inhibit-read-only t))
@@ -1005,13 +975,10 @@ when multi-line constructs (docstrings, block comments) start
 above the visible window.
 
 Inhibits modification hooks to prevent jit-lock from scanning the
-full buffer on each delta.  Skips the delete+reinsert cycle when the
-visible tail content is unchanged from the previous delta."
+full buffer on each delta."
   (when (and pi-coding-agent--pending-tool-overlay
              raw-text
              (not (string-empty-p raw-text)))
-    ;; Always sync fontify buffer (incremental, cheap) regardless of
-    ;; whether the display will update — the buffer must stay current.
     (when lang
       (pi-coding-agent--fontify-sync raw-text lang))
     (let* ((tail-result
@@ -1033,52 +1000,38 @@ visible tail content is unchanged from the previous delta."
             (or has-hidden
                 (and truncation
                      (> (plist-get truncation :hidden-lines) 0))))
-           ;; Compare plain text to cached value — skip if unchanged.
-           ;; Property-stripped comparison is correct: the text determines
-           ;; whether the preview changed.  Font-lock properties may shift
-           ;; cosmetically but that's not worth a full redraw.
-           (display-text (substring-no-properties display-content))
-           (cache-key (if show-hidden-indicator
-                         (concat "H:" display-text)
-                       display-text))
-           (last-tail (overlay-get pi-coding-agent--pending-tool-overlay
-                                   'pi-coding-agent-last-tail)))
-      ;; Skip display when the visible tail is unchanged
-      (unless (or (string-empty-p display-content)
-                  (equal cache-key last-tail))
-        (let ((inhibit-read-only t)
-              (inhibit-modification-hooks t))
-          (pi-coding-agent--with-scroll-preservation
-            (save-excursion
-              (let* ((ov-end (overlay-end pi-coding-agent--pending-tool-overlay))
-                     (header-end (overlay-get pi-coding-agent--pending-tool-overlay
-                                              'pi-coding-agent-header-end)))
-                ;; Delete previous streaming content (everything after header)
-                (when (and header-end (< header-end ov-end))
-                  (delete-region header-end ov-end))
-                ;; Insert new streaming content
-                (goto-char (overlay-end pi-coding-agent--pending-tool-overlay))
-                (when show-hidden-indicator
-                  (insert (propertize "... (earlier output)\n"
-                                      'face 'pi-coding-agent-collapsed-indicator)))
-                (let ((content-start (point)))
-                  (insert (if lang
-                              display-content
-                            (pi-coding-agent--render-tool-content
-                             display-content nil)))
-                  (insert "\n")
-                  ;; Mark pre-fontified content so jit-lock won't override
-                  ;; our syntax faces with gfm-mode faces on redisplay.
-                  ;; Also layer markdown-code-face underneath so the text
-                  ;; uses fixed-pitch font, matching completed code blocks.
-                  (when lang
-                    (put-text-property content-start (point)
-                                       'fontified t)
-                    (add-face-text-property content-start (point)
-                                            'markdown-code-face t))))))
-          ;; Cache the displayed content for next comparison
-          (overlay-put pi-coding-agent--pending-tool-overlay
-                       'pi-coding-agent-last-tail cache-key))))))
+           (inhibit-read-only t)
+           (inhibit-modification-hooks t))
+      ;; Nothing to display yet (e.g., first delta before any newline)
+      (unless (string-empty-p display-content)
+        (pi-coding-agent--with-scroll-preservation
+          (save-excursion
+            (let* ((ov-end (overlay-end pi-coding-agent--pending-tool-overlay))
+                   (header-end (overlay-get pi-coding-agent--pending-tool-overlay
+                                            'pi-coding-agent-header-end)))
+              ;; Delete previous streaming content (everything after header)
+              (when (and header-end (< header-end ov-end))
+                (delete-region header-end ov-end))
+              ;; Insert new streaming content
+              (goto-char (overlay-end pi-coding-agent--pending-tool-overlay))
+              (when show-hidden-indicator
+                (insert (propertize "... (earlier output)\n"
+                                    'face 'pi-coding-agent-collapsed-indicator)))
+              (let ((content-start (point)))
+                (insert (if lang
+                            display-content
+                          (pi-coding-agent--render-tool-content
+                           display-content nil)))
+                (insert "\n")
+                ;; Mark pre-fontified content so jit-lock won't override
+                ;; our syntax faces with gfm-mode faces on redisplay.
+                ;; Also layer markdown-code-face underneath so the text
+                ;; uses fixed-pitch font, matching completed code blocks.
+                (when lang
+                  (put-text-property content-start (point)
+                                     'fontified t)
+                  (add-face-text-property content-start (point)
+                                          'markdown-code-face t))))))))))
 
 (defun pi-coding-agent--display-tool-update (partial-result)
   "Display PARTIAL-RESULT as streaming output in pending tool overlay.
@@ -1091,27 +1044,10 @@ Extracts text from content blocks and delegates to
       (pi-coding-agent--display-tool-streaming-text
        raw-output pi-coding-agent-bash-preview-lines))))
 
-(defun pi-coding-agent--markdown-fence-delimiter (content)
-  "Return a markdown fence delimiter safe for CONTENT.
-Uses triple backticks by default.  If CONTENT contains triple-backtick
-runs, uses a tilde fence longer than any tilde run in CONTENT."
-  (let ((text (or content "")))
-    (if (string-match-p "```+" text)
-        (let ((max-tilde-run 0)
-              (pos 0))
-          (while (string-match "~+" text pos)
-            (setq max-tilde-run
-                  (max max-tilde-run
-                       (- (match-end 0) (match-beginning 0))))
-            (setq pos (match-end 0)))
-          (make-string (max 3 (1+ max-tilde-run)) ?~))
-      "```")))
-
 (defun pi-coding-agent--wrap-in-src-block (content lang)
   "Wrap CONTENT in a markdown fenced code block with LANG.
 Returns markdown string for syntax highlighting."
-  (let ((fence (pi-coding-agent--markdown-fence-delimiter content)))
-    (format "%s%s\n%s\n%s" fence (or lang "") content fence)))
+  (format "```%s\n%s\n```" (or lang "") content))
 
 (defun pi-coding-agent--render-tool-content (content lang)
   "Render CONTENT with optional syntax highlighting for LANG.
@@ -1121,28 +1057,9 @@ Returns the rendered string."
       (pi-coding-agent--wrap-in-src-block content lang)
     (propertize content 'face 'pi-coding-agent-tool-output)))
 
-(defun pi-coding-agent--fontify-get-buffer (lang)
-  "Return the fontification cache buffer for LANG in the current session.
-Looks up the buffer-local `pi-coding-agent--fontify-buffers' hash table.
-Returns nil if no buffer exists for LANG.  Removes stale entries
-for buffers that have been killed externally."
-  (when pi-coding-agent--fontify-buffers
-    (let ((buf (gethash lang pi-coding-agent--fontify-buffers)))
-      (cond
-       ((null buf) nil)
-       ((buffer-live-p buf) buf)
-       (t (remhash lang pi-coding-agent--fontify-buffers) nil)))))
-
-(defun pi-coding-agent--fontify-get-or-create-buffer (lang)
-  "Return or create the fontification cache buffer for LANG.
-Uses the buffer-local hash table to track per-session buffers.
-Returns nil if called outside a chat buffer (no hash table)."
-  (when pi-coding-agent--fontify-buffers
-    (or (pi-coding-agent--fontify-get-buffer lang)
-        (let ((buf (generate-new-buffer
-                    (format " *pi-fontify:%s:%s*" lang (buffer-name)))))
-          (puthash lang buf pi-coding-agent--fontify-buffers)
-          buf))))
+(defsubst pi-coding-agent--fontify-buffer-name (lang)
+  "Return the name of the fontification cache buffer for LANG."
+  (format " *pi-fontify:%s*" lang))
 
 (defun pi-coding-agent--fontify-reset (args)
   "Clear the fontification buffer for the language implied by ARGS.
@@ -1151,7 +1068,7 @@ buffer, preventing stale content from a previous call from being
 treated as a matching prefix during incremental sync."
   (when-let* ((lang (pi-coding-agent--path-to-language
                      (pi-coding-agent--tool-path args)))
-              (buf (pi-coding-agent--fontify-get-buffer lang)))
+              (buf (get-buffer (pi-coding-agent--fontify-buffer-name lang))))
     (with-current-buffer buf
       (erase-buffer))))
 
@@ -1164,8 +1081,9 @@ incorrect keyword matching on partial tokens.
 The buffer always accumulates content regardless of whether the
 language mode is available, so `pi-coding-agent--fontify-buffer-tail'
 can extract the tail even for languages without an installed mode."
-  (condition-case err
-      (when-let* ((buf (pi-coding-agent--fontify-get-or-create-buffer lang)))
+  (condition-case nil
+      (let ((buf (get-buffer-create
+                  (pi-coding-agent--fontify-buffer-name lang))))
         (with-current-buffer buf
           ;; Activate language mode once (best-effort for fontification).
           (let ((mode (and lang (markdown-get-lang-mode lang))))
@@ -1199,8 +1117,7 @@ can extract the tail even for languages without an installed mode."
               (ignore-errors
                 (font-lock-default-fontify-region
                  (point-min) (point-max) nil)))))))
-    (error
-     (message "pi-coding-agent: fontify-sync error for %s: %S" lang err))))
+    (error nil)))
 
 (defun pi-coding-agent--fontify-buffer-tail (lang n)
   "Extract last N non-blank complete lines from the LANG fontification buffer.
@@ -1218,7 +1135,7 @@ Returns (CONTENT . HAS-HIDDEN) where CONTENT is a string with text
 properties preserved.  HAS-HIDDEN is non-nil when earlier lines
 exist above the returned tail.
 Returns nil if the buffer doesn't exist or has no complete lines."
-  (let ((buf (pi-coding-agent--fontify-get-buffer lang)))
+  (let ((buf (get-buffer (pi-coding-agent--fontify-buffer-name lang))))
     (when (and buf (> (buffer-size buf) 0))
       (with-current-buffer buf
         ;; Find end of last complete line (just before the trailing
@@ -1245,17 +1162,6 @@ Returns nil if the buffer doesn't exist or has no complete lines."
               (cons (mapconcat #'identity line-strings "\n")
                     (> (point) (point-min))))))))))
 
-(defun pi-coding-agent--kill-fontify-buffers ()
-  "Kill all fontification cache buffers for the current session.
-Iterates the buffer-local `pi-coding-agent--fontify-buffers' hash table
-and kills each buffer, then clears the table."
-  (when pi-coding-agent--fontify-buffers
-    (maphash (lambda (_lang buf)
-               (when (buffer-live-p buf)
-                 (kill-buffer buf)))
-             pi-coding-agent--fontify-buffers)
-    (clrhash pi-coding-agent--fontify-buffers)))
-
 (defun pi-coding-agent--display-tool-end (tool-name args content details is-error)
   "Display result for TOOL-NAME and update overlay face.
 ARGS contains tool arguments, CONTENT is a list of content blocks.
@@ -1476,68 +1382,18 @@ Diff format: [+-] LINENUM content (e.g., '+ 7     code' or '-12     code')."
     (when (looking-at "^[+-] *\\([0-9]+\\)")
       (string-to-number (match-string 1)))))
 
-(defun pi-coding-agent--fence-line-info-at-point ()
-  "Return fence info for current line, or nil when not on a fence line.
-Return value is plist `(:char CHAR :len LEN :trailing TEXT)'.
-TEXT is everything after the fence run on this line.
-Only recognizes fences indented by at most three spaces."
-  (save-excursion
-    (beginning-of-line)
-    (let ((indent-start (point)))
-      (skip-chars-forward " ")
-      (let ((indent (- (point) indent-start))
-            (char (char-after)))
-        (when (and (<= indent 3)
-                   (memq char '(?` ?~)))
-          (let ((start (point)))
-            (skip-chars-forward (char-to-string char))
-            (let ((len (- (point) start)))
-              (when (>= len 3)
-                (list :char char
-                      :len len
-                      :trailing (buffer-substring-no-properties
-                                 (point) (line-end-position)))))))))))
-
-(defun pi-coding-agent--fence-closing-line-p (fence line-info)
-  "Return non-nil when LINE-INFO closes FENCE.
-FENCE and LINE-INFO are plists from
-`pi-coding-agent--fence-line-info-at-point'."
-  (and line-info
-       (= (plist-get line-info :char) (plist-get fence :char))
-       (>= (plist-get line-info :len) (plist-get fence :len))
-       (string-match-p "^[ \t]*$" (plist-get line-info :trailing))))
-
-(defun pi-coding-agent--code-block-line-at-point (&optional start-pos)
+(defun pi-coding-agent--code-block-line-at-point ()
   "Return line number within code block content at point.
-Supports both backtick and tilde fenced blocks.  Returns nil unless
-point is on a content line inside an open fenced block.
-When START-POS is non-nil, parse fences starting from that position."
+Searches backward for opening fence (```), counts lines from there.
+Returns nil if not inside a code block or if on the fence line itself."
   (save-excursion
-    (let* ((target-line (line-number-at-pos))
-           (scan-start (or start-pos (point-min)))
-           (open-fence nil)
-           (open-line nil)
-           (result nil)
-           line-no)
-      (goto-char scan-start)
+    (let ((orig-line (line-number-at-pos)))
       (beginning-of-line)
-      (setq line-no (line-number-at-pos))
-      (while (and (<= line-no target-line) (not result))
-        (let ((line-info (pi-coding-agent--fence-line-info-at-point)))
-          (cond
-           (open-fence
-            (cond
-             ((pi-coding-agent--fence-closing-line-p open-fence line-info)
-              (setq open-fence nil)
-              (setq open-line nil))
-             ((= line-no target-line)
-              (setq result (- line-no open-line)))))
-           (line-info
-            (setq open-fence line-info)
-            (setq open-line line-no))))
-        (forward-line 1)
-        (setq line-no (1+ line-no)))
-      result)))
+      (when (re-search-backward "^```" nil t)
+        (let ((fence-line (line-number-at-pos)))
+          ;; Only return line number if we're after the fence line
+          (when (> orig-line fence-line)
+            (- orig-line fence-line)))))))
 
 (defun pi-coding-agent--tool-line-at-point (overlay)
   "Calculate file line number at point for tool OVERLAY.
@@ -1560,7 +1416,7 @@ For read/write: count lines from header + apply line-map for stripped blanks."
            (when (and (>= map-index 0) (< map-index (length line-map)))
              (+ (aref line-map map-index) (1- offset))))))
      ;; Fallback: code block position + offset (for expanded view or no line-map)
-     (when-let ((block-line (pi-coding-agent--code-block-line-at-point header-end)))
+     (when-let ((block-line (pi-coding-agent--code-block-line-at-point)))
        (+ block-line (1- offset)))
      ;; Last fallback to line 1
      1)))
@@ -1805,57 +1661,14 @@ correct context, not blocks from earlier in the buffer."
         (funcall orig-fun prop (if lim (max lim limit) limit)))
     (funcall orig-fun prop lim)))
 
-(defun pi-coding-agent--restore-tool-properties (beg end)
-  "Strip markdown text properties from the pending tool overlay in BEG..END.
-Removes properties that gfm-mode fontification applies to markup
-patterns in tool output:
-- `display' (\"\"): hides # in headings
-- `invisible' (markdown-markup): hides ** __ and heading markup
-- `font-lock-multiline': causes fontification region extensions
-- `face': overrides tool faces with markdown heading/bold faces
-Restores intended faces for both the header and content regions."
-  (when-let* ((ov pi-coding-agent--pending-tool-overlay)
-              (ov-start (overlay-start ov))
-              (ov-end (overlay-end ov))
-              (header-end-marker (overlay-get ov 'pi-coding-agent-header-end))
-              (header-end (marker-position header-end-marker)))
-    (when (and (< beg ov-end) (> end ov-start))
-      (let ((inhibit-read-only t))
-        ;; Header: restore face from font-lock-face (varies per span)
-        (let ((hdr-beg (max beg ov-start))
-              (hdr-end (min end header-end)))
-          (when (< hdr-beg hdr-end)
-            (remove-text-properties
-             hdr-beg hdr-end
-             '(display nil invisible nil font-lock-multiline nil))
-            (let ((pos hdr-beg))
-              (while (< pos hdr-end)
-                (let* ((fl-face (get-text-property pos 'font-lock-face))
-                       (next (or (next-single-property-change
-                                  pos 'font-lock-face nil hdr-end)
-                                 hdr-end)))
-                  (when fl-face
-                    (put-text-property pos next 'face fl-face))
-                  (setq pos next))))
-            (put-text-property hdr-beg hdr-end 'fontified t)))
-        ;; Content: uniform tool-output face
-        (let ((cnt-beg (max beg header-end))
-              (cnt-end (min end ov-end)))
-          (when (< cnt-beg cnt-end)
-            (remove-text-properties
-             cnt-beg cnt-end
-             '(display nil invisible nil font-lock-multiline nil))
-            (put-text-property cnt-beg cnt-end 'face
-                               'pi-coding-agent-tool-output)
-            (put-text-property cnt-beg cnt-end 'fontified t)))))))
-
 (defun pi-coding-agent--fontify-streaming-region ()
   "Fontify newly streamed message text incrementally.
 Called by idle timer during streaming.  Only fontifies message text
 that hasn't been fontified yet, tracked via the variable
 `pi-coding-agent--last-fontified-pos'.  Skips the pending tool
-overlay region to avoid applying gfm-mode faces to tool content
-via `font-lock-ensure' (which is not cleaned up by jit-lock)."
+overlay region — tool content is pre-fontified in a temp buffer
+and marked with the `fontified' property to prevent jit-lock from
+overriding it with gfm-mode faces."
   (when (and pi-coding-agent--message-start-marker
              pi-coding-agent--streaming-marker
              (marker-position pi-coding-agent--message-start-marker)
