@@ -54,7 +54,6 @@
 (declare-function pi-coding-agent-toggle-tool-section "pi-coding-agent-render")
 (declare-function pi-coding-agent-visit-file "pi-coding-agent-render")
 (declare-function pi-coding-agent--cleanup-on-kill "pi-coding-agent-render")
-(declare-function pi-coding-agent--restore-tool-properties "pi-coding-agent-render")
 
 ;; pi-coding-agent-input.el (input buffer commands)
 (declare-function pi-coding-agent-quit "pi-coding-agent-input")
@@ -155,27 +154,6 @@ When phscroll is not available, tables wrap like other content."
   :type 'boolean
   :group 'pi-coding-agent)
 
-(defcustom pi-coding-agent-input-markdown-highlighting nil
-  "Whether to enable GFM syntax highlighting in the input buffer.
-When non-nil, the input buffer gets GitHub Flavored Markdown
-highlighting (bold, italic, code spans, fenced blocks).  When nil,
-the input buffer uses plain `text-mode'.
-
-Takes effect for new sessions; existing input buffers keep their mode."
-  :type 'boolean
-  :group 'pi-coding-agent)
-
-(defcustom pi-coding-agent-copy-raw-markdown nil
-  "Whether to copy raw markdown from the chat buffer.
-When non-nil, copy commands (`kill-ring-save', `kill-region') preserve
-raw markdown — bold markers (**), backticks, code fences, and setext
-underlines are kept.  Useful for pasting into docs, Slack, or other
-markdown-aware contexts.
-
-When nil (the default), only the visible text is copied."
-  :type 'boolean
-  :group 'pi-coding-agent)
-
 ;;;; Faces
 
 (defface pi-coding-agent-timestamp
@@ -375,36 +353,6 @@ This makes wrapped lines show the blockquote indicator."
                        'wrap-prefix pi-coding-agent--blockquote-wrap-prefix)
     t))
 
-;;;; Copy Visible Text
-
-(defun pi-coding-agent--visible-text (beg end)
-  "Return visible text between BEG and END, stripping hidden markup.
-Skips characters with `invisible' property matching `buffer-invisibility-spec'
-and characters with `display' property equal to the empty string."
-  (let ((result nil)
-        (pos beg))
-    (while (< pos end)
-      (let* ((inv (get-text-property pos 'invisible))
-             (disp (get-text-property pos 'display))
-             (next (min (next-single-char-property-change pos 'invisible nil end)
-                        (next-single-char-property-change pos 'display nil end))))
-        (cond
-         ((and inv (invisible-p inv)) nil)
-         ((equal disp "") nil)
-         (t (push (buffer-substring-no-properties pos next) result)))
-        (setq pos next)))
-    (apply #'concat (nreverse result))))
-
-(defun pi-coding-agent--filter-buffer-substring (beg end &optional delete)
-  "Filter function for `filter-buffer-substring-function' in chat buffers.
-When `pi-coding-agent-copy-raw-markdown' is nil, returns only visible
-text between BEG and END.  If DELETE is non-nil, also removes the region.
-Otherwise delegates to the default filter."
-  (if pi-coding-agent-copy-raw-markdown
-      (buffer-substring--filter beg end delete)
-    (prog1 (pi-coding-agent--visible-text beg end)
-      (when delete (delete-region beg end)))))
-
 (define-derived-mode pi-coding-agent-chat-mode gfm-mode "Pi-Chat"
   "Major mode for displaying pi conversation.
 Derives from `gfm-mode' for syntax highlighting of code blocks.
@@ -417,11 +365,7 @@ This is a read-only buffer showing the conversation history."
   ;; Hide markdown markup (**, `, ```) for cleaner display
   (setq-local markdown-hide-markup t)
   (add-to-invisibility-spec 'markdown-markup)
-  ;; Strip hidden markup from copy operations (M-w, C-w)
-  (setq-local filter-buffer-substring-function
-              #'pi-coding-agent--filter-buffer-substring)
   (setq-local pi-coding-agent--tool-args-cache (make-hash-table :test 'equal))
-  (setq-local pi-coding-agent--fontify-buffers (make-hash-table :test 'equal))
   ;; Disable hl-line-mode: its post-command-hook overlay update causes
   ;; scroll oscillation in buffers with invisible text + variable heights.
   (setq-local global-hl-line-mode nil)
@@ -433,9 +377,6 @@ This is a read-only buffer showing the conversation history."
   ;; Add wrap-prefix to blockquotes so wrapped lines show the indicator
   (font-lock-add-keywords nil '((pi-coding-agent--fontify-blockquote-wrap-prefix)) 'append)
 
-  ;; Run after font-lock to undo markdown damage in tool overlays.
-  (jit-lock-register #'pi-coding-agent--restore-tool-properties)
-
   ;; Enable phscroll for horizontal table scrolling, offer install if missing
   (pi-coding-agent--maybe-install-phscroll)
   (when (pi-coding-agent--phscroll-available-p)
@@ -553,12 +494,6 @@ Used to suppress ATX heading transforms inside code.")
   "Non-nil when streaming inside a thinking block.
 Used to add blockquote prefix to each line.")
 
-(defvar-local pi-coding-agent--thinking-marker nil
-  "Marker for insertion point inside the current thinking block.
-Unlike `pi-coding-agent--streaming-marker', this marker stays anchored
-in thinking text when other content blocks (for example, tool headers)
-interleave during streaming.")
-
 (defvar-local pi-coding-agent--line-parse-state 'line-start
   "Parsing state for current line during streaming.
 Values:
@@ -631,13 +566,6 @@ Enables dedup guard in tool_execution_start to skip overlay creation
 when the overlay was already created by the streaming event path.
 Set at toolcall_start, consumed and cleared at tool_execution_start.")
 
-(defvar-local pi-coding-agent--fontify-buffers nil
-  "Hash table mapping language strings to fontification cache buffers.
-Each chat buffer tracks its own fontify buffers so parallel sessions
-writing the same language don't corrupt each other's syntax state.
-Initialized in `pi-coding-agent-chat-mode'; cleaned up by
-`pi-coding-agent--kill-fontify-buffers' when the session ends.")
-
 (defvar-local pi-coding-agent--assistant-header-shown nil
   "Non-nil if Assistant header has been shown for current prompt.
 Used to avoid duplicate headers during retry sequences.")
@@ -889,7 +817,7 @@ Displays warnings for missing dependencies."
 
 ;;;; Startup Header
 
-(defconst pi-coding-agent-version "1.3.2"
+(defconst pi-coding-agent-version "1.2.6"
   "Version of pi-coding-agent.")
 
 (defun pi-coding-agent--get-pi-coding-agent-version ()
